%{
#define YYDEBUG 1
#define YYSTYPE SCM
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "scm-yacc-glue.h"
int yylex ();
void ready_lexer (char *expression);
void yyerror (char *s);
SCM parse_return;
%}

%left COL COLEQU
%left LP RP LCURLY RCURLY BLOCKSTART BLOCKEND
%left SEMICOL
%left DO OD
%left BOX
%left RARROW LARROW
%left VAR INT
%left EQUIV DISCREP
%left IMPLIES FOLLOWS
%left OR AND
%left LT GT GE LE EQ NE
%left PLUS
%left MAXOP MINOP TIMES DIV DIVIDE
%left MINUS NEG COMMA
%left DOT
%left NUMBER
%left BOOLEAN
%left VARIABLE
%left QUANT


%% /* Grammar rules and actions follow */

input: input exp     { parse_return = append_to_exp_list (parse_return, $2); }
     |
     ;

exp: NUMBER          { $$ = $1; }
   | BOOLEAN         { $$ = $1; }
   | VARIABLE        { $$ = $1; }
   | INT             { $$ = $1; }
   | exp OR exp      { $$ = new_infix_exp ("\\/", $1, $3); }
   | exp AND exp     { $$ = new_infix_exp ("/\\", $1, $3); }
   | exp PLUS exp    { $$ = new_infix_exp ("+", $1, $3); }
   | exp TIMES exp   { $$ = new_infix_exp ("*", $1, $3); }
   | exp DIV exp     { $$ = new_infix_exp ("div", $1, $3); }
   | exp DIVIDE exp  { $$ = new_infix_exp ("/", $1, $3); }
   | exp RARROW exp  { $$ = new_infix_exp ("-->", $1, $3); }
   | exp LARROW exp  { $$ = new_infix_exp ("<--", $1, $3); }
   | exp EQUIV exp   { $$ = new_infix_exp ("<==>", $1, $3); }
   | exp DISCREP exp { $$ = new_infix_exp ("<=!=>", $1, $3); }
   | exp LT exp      { $$ = new_infix_exp ("<", $1, $3); }
   | exp GT exp      { $$ = new_infix_exp (">", $1, $3); }
   | exp GE exp      { $$ = new_infix_exp (">=", $1, $3); }
   | exp LE exp      { $$ = new_infix_exp ("<=", $1, $3); }
   | exp EQ exp      { $$ = new_infix_exp ("=", $1, $3); }
   | exp NE exp      { $$ = new_infix_exp ("!=", $1, $3); }
   | exp MAXOP exp   { $$ = new_infix_exp ("max", $1, $3); }
   | exp MINOP exp   { $$ = new_infix_exp ("min", $1, $3); }
   | NEG exp         { $$ = new_unary_exp ("~", $2); }
   | MINUS exp       { $$ = new_unary_exp ("-", $2); }
   | exp DOT exp     { $$ = new_infix_exp (".", $1, $3); }
   | LP exp RP       { $$ = $2 }
   | BLOCKSTART exp BLOCKEND
                     { $$ = new_unary_exp ("|[ ]|", $2); }
   | exp COLEQU exp  { $$ = new_infix_exp (":=", $1, $3); }
   | exp SEMICOL exp { $$ = new_infix_exp (";", $1, $3); }
   | exp COMMA exp   { $$ = new_infix_exp (",", $1, $3); }
   | DO exp OD       { $$ = new_unary_exp ("do", $2); }
   | exp IMPLIES exp { $$ = new_infix_exp ("==>", $1, $3); }
   | exp FOLLOWS exp { $$ = new_infix_exp ("<==", $1, $3); }
   | exp BOX exp     { $$ = new_infix_exp ("[]", $1, $3); }
   | VAR exp         { $$ = new_unary_exp ("var", $2); }
   | LP QUANT exp COL exp COL exp RP
                     { $$ = new_quant_exp ($2, $3, $5, $7); }
   | LP QUANT exp COL COL exp RP
                     { $$ = new_quant_exp ($2, $3, new_boolean_exp (1), $6); }
   | LP QUANT exp COL exp COL RP
                     { $$ = new_quant_exp ($2, $3, $5, new_boolean_exp (0)); }
   | LP QUANT exp COL COL RP
                     { $$ = new_quant_exp ($2, $3, new_boolean_exp (1),
					   new_boolean_exp (0)); }
   ;

%%


void yyerror (char *s)
{
    printf ("%s\n", s);
}


SCM parse_gcl (char *expressions)
{
  parse_return = new_exp_list ();

  ready_lexer (expressions);
  yyparse ();

  return parse_return;
}
